from pandas import read_csv
from pandas import to_datetime
from pandas import DataFrame
from prophet import Prophet
from matplotlib import pyplot
import json
import datetime
from prophet.serialize import model_to_json, model_from_json


def train_fbp(company_name):
    # net profit
    path = 'impact_of_company_financial/data/' + company_name + '_net_profit_margin.csv'
    df = read_csv(path, header=0)

    df.columns = ['ds', 'y']
    df['ds'] = to_datetime(df['ds'])

    model = Prophet()

    model.fit(df)

    future = list()
    for i in range(1, 13):
        date = '2022-%02d' % i
        future.append([date])
    future = DataFrame(future)
    future.columns = ['ds']
    future['ds'] = to_datetime(future['ds'])

    forecast = model.predict(future)

    print(forecast['yhat'][-7:])

    with open('impact_of_company_financial/trained_models/' + company_name + '_net_profit_margin.json', 'w') as fout:
        json.dump(model_to_json(model), fout)

    # earning per share
    path = 'data/' + company_name + '_earnings_per_share.csv'
    df = read_csv(path, header=0)

    df.columns = ['ds', 'y']
    df['ds'] = to_datetime(df['ds'])

    model = Prophet()

    model.fit(df)

    future = list()
    for i in range(1, 13):
        date = '2022-%02d' % i
        future.append([date])
    future = DataFrame(future)
    future.columns = ['ds']
    future['ds'] = to_datetime(future['ds'])

    forecast = model.predict(future)

    print(forecast['yhat'][-7:])

    with open('trained_models/' + company_name + '_earnings_per_share.json', 'w') as fout:
        json.dump(model_to_json(model), fout)


def forecast_fbp(company_name, no_of_months):
    path_net_profit_margin = 'impact_of_company_financial/data/' + company_name + '_net_profit_margin.csv'
    df_net_profit_margin = read_csv(path_net_profit_margin, header=0)

    path_earnings_per_share = 'impact_of_company_financial/data/' + company_name + '_earnings_per_share.csv'
    df_earnings_per_share = read_csv(path_earnings_per_share, header=0)

    df_net_profit_margin.columns = ['ds', 'y']
    df_net_profit_margin['ds'] = to_datetime(df_net_profit_margin['ds'])

    df_earnings_per_share.columns = ['ds', 'y']
    df_earnings_per_share['ds'] = to_datetime(df_earnings_per_share['ds'])

    model_net_profit_margin = Prophet()
    model_earnings_per_share = Prophet()

    model_net_profit_margin.fit(df_net_profit_margin)
    model_earnings_per_share.fit(df_earnings_per_share)

    # with open('trained_models/' + str(company_name) + '_net_profit_margin.json', 'r') as fin:
    #     model_net_profit_margin = model_from_json(fin.read())
    #
    # with open('trained_models/' + str(company_name) + '_earnings_per_share.json', 'r') as fin:
    #     model_earnings_per_share = model_from_json(fin.read())

    future = list()
    future_dates = []
    today = datetime.datetime.today()

    for i in range(no_of_months):
        today = today + datetime.timedelta(days=30)
        future.append([today.strftime('%Y-%m')])
        future_dates.append(today.strftime('%Y-%m'))
    future = DataFrame(future)
    future.columns = ['ds']
    future['ds'] = to_datetime(future['ds'])

    forecast_net_profit_margin = model_net_profit_margin.predict(future)
    forecast_earnings_per_share = model_earnings_per_share.predict(future)

    data_net_profit_margin = []
    data_earnings_per_share = []

    for i in forecast_net_profit_margin['yhat'][-no_of_months:]:
        data_net_profit_margin.append(round(i, 2))

    for i in forecast_earnings_per_share['yhat'][-no_of_months:]:
        data_earnings_per_share.append(round(i, 2))

    return data_net_profit_margin, data_earnings_per_share, future_dates


# train_fbp('ASIA_ASSET_FINANCE_PLC')
# x,y  = forecast_fbp('ASIA_ASSET_FINANCE_PLC',5)

# print(x,y)